/*******************************************************************
    A telegram bot for your ESP8266 that controls the 
    onboard LED. The LED in this example is active low.

    Parts:
    D1 Mini ESP8266 * - http://s.click.aliexpress.com/e/uzFUnIe
    (or any ESP8266 board)

      = Affilate

    If you find what I do useful and would like to support me,
    please consider becoming a sponsor on Github
    https://github.com/sponsors/witnessmenow/


    Written by Brian Lough
    YouTube: https://www.youtube.com/brianlough
    Tindie: https://www.tindie.com/stores/brianlough/
    Twitter: https://twitter.com/witnessmenow
 *******************************************************************/

#include <ESP8266WiFi.h>
#include <WiFiClientSecure.h>
#include <UniversalTelegramBot.h>

// Wifi network station credentials
#define WIFI_SSID "xxxxxx"
#define WIFI_PASSWORD "xxxxx"
// Telegram BOT Token (Get from Botfather)
#define BOT_TOKEN "0000000000:XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX"

const unsigned long BOT_MTBS = 1000; // mean time between scan messages

X509List cert(TELEGRAM_CERTIFICATE_ROOT);
WiFiClientSecure secured_client;
UniversalTelegramBot bot(BOT_TOKEN, secured_client);
unsigned long bot_lasttime; // last time messages' scan has been done

const int ledPinVerde = 0;
const int ledPinVermelho = 2;
int ledStatusVerde = 0;
int ledStatusVermelho = 0;

void handleNewMessages(int numNewMessages)
{
  Serial.print("Nova mensagem ");
  Serial.println(numNewMessages);
  String msg;
  
  for (int i = 0; i < numNewMessages; i++)
  {
    String chat_id = bot.messages[i].chat_id;
    String text = bot.messages[i].text;

    String from_name = bot.messages[i].from_name;
    if (from_name == "")
      from_name = "Guest";

    if (text == "/VermelhoON") {
        digitalWrite(ledPinVermelho, HIGH); 
        ledStatusVermelho = 1;
        bot.sendMessage(chat_id, "Led Vermelho Ligado", "");
    }
    else if (text == "/VerdeON") {
        digitalWrite(ledPinVerde, HIGH); 
        ledStatusVerde = 1;
        bot.sendMessage(chat_id, "Led Verde Ligado", "");
    }
    else if (text == "/Liga") {
        digitalWrite(ledPinVermelho, HIGH); 
        digitalWrite(ledPinVerde, HIGH); 
        ledStatusVermelho = 1;
        ledStatusVerde = 1;
        msg = "";
        msg += "\n Led Vermelho Ligado"; 
        msg += "\n Led Verde Ligado";
        bot.sendMessage(chat_id, msg, "");
    }
    else if (text == "/VermelhoOFF") {
        digitalWrite(ledPinVermelho, LOW); 
        ledStatusVermelho = 0;
        bot.sendMessage(chat_id, "Led Vermelho Desligado", "");
    }
    else if (text == "/VerdeOFF") {
        digitalWrite(ledPinVerde, LOW); 
        ledStatusVerde = 0;
        bot.sendMessage(chat_id, "Led Verde Desligado", "");
    }
    else if (text == "/Desliga") {
        digitalWrite(ledPinVermelho, LOW);
        digitalWrite(ledPinVerde, LOW);  
        ledStatusVermelho = 0;
        ledStatusVerde = 0;
        msg = "";
        msg += "Led Vermelho Desligado";
        msg += "Led Verde Desligado";
        bot.sendMessage(chat_id, msg, "");
    }
    else if (text == "/Situacao") {
        msg = "";
        if (ledStatusVermelho) {
          msg += "\n Led Vermelho Ligado";
        }
        else {
          msg += "\n Led Vermelho Desligado";
        }
        if (ledStatusVerde) {
          msg += "\n Led Verde Ligado";
        }
        else {
          msg += "\n Led Verde Desligado";
        }
        bot.sendMessage(chat_id, msg, "");
    }
    else {
        msg = "Benvindo, " + from_name + ".\n";
        msg += "Segue lista de comandos:\n\n";
        msg += "/VermelhoON : Ligar LED Vermelho\n";
        msg += "/VerdeON : Ligar LED Verde\n";
        msg += "/Liga : Ligar todos LEDs\n\n";
        msg += "/VermelhoOFF : Desligar LED Vermelho\n";
        msg += "/VerdeOFF : Desligar LED Verde\n";
        msg += "/Desliga : Desligar todos LEDs\n\n";
        msg += "/Situacao : Mostra as situações dos LEDs\n";
        bot.sendMessage(chat_id, msg, "Markdown");
    }
    
    /*if (text == "/start")
    {
      String welcome = "Welcome to Universal Arduino Telegram Bot library, " + from_name + ".\n";
      welcome += "This is Flash Led Bot example.\n\n";
      welcome += "/ledon : to switch the Led ON\n";
      welcome += "/ledoff : to switch the Led OFF\n";
      welcome += "/status : Returns current status of LED\n";
      bot.sendMessage(chat_id, welcome, "Markdown");
    }*/
  }
}


void setup()
{
  Serial.begin(115200);
  Serial.println();

  pinMode(ledPinVerde, OUTPUT); // initialize digital ledPin as an output.
  pinMode(ledPinVermelho, OUTPUT); // initialize digital ledPin as an output.
  delay(10);
  digitalWrite(ledPinVerde, LOW); // initialize pin as off (active LOW)
  digitalWrite(ledPinVermelho, LOW); // initialize pin as off (active LOW)

  // attempt to connect to Wifi network:
  configTime(0, 0, "pool.ntp.org");      // get UTC time via NTP
  secured_client.setTrustAnchors(&cert); // Add root certificate for api.telegram.org
  Serial.print("Connecting to Wifi SSID ");
  Serial.print(WIFI_SSID);
  WiFi.begin(WIFI_SSID, WIFI_PASSWORD);
  while (WiFi.status() != WL_CONNECTED)
  {
    Serial.print(".");
    delay(500);
  }
  Serial.print("\nWiFi connected. IP address: ");
  Serial.println(WiFi.localIP());

  // Check NTP/Time, usually it is instantaneous and you can delete the code below.
  Serial.print("Retrieving time: ");
  time_t now = time(nullptr);
  while (now < 24 * 3600)
  {
    Serial.print(".");
    delay(100);
    now = time(nullptr);
  }
  Serial.println(now);
}

void loop()
{
  if (millis() - bot_lasttime > BOT_MTBS)
  {
    int numNewMessages = bot.getUpdates(bot.last_message_received + 1);

    while (numNewMessages)
    {
      Serial.println("got response");
      handleNewMessages(numNewMessages);
      numNewMessages = bot.getUpdates(bot.last_message_received + 1);
    }

    bot_lasttime = millis();
  }
}
